import aiohttp
from .model import *


class Client:
    """Main Airi Client class to use this library."""

    def __init__(self, token: str) -> str:
        """
        airi.py client initialization

        :param token: API token
        :type token: str
        """
        self._header = {"Auth": token}
        self._baseurl = "https://airi.kyoyo.me/api/"
        self._testurl = "https://airi.kyoyo.me/api/password"

    async def _endpoint(self, endpoint: str) -> str:
        """
        NOT meant to be used in any way by a developer/user


        Parameter:-
            endpoint (`str`): endpoint name
        """

        async with aiohttp.ClientSession() as session:
            response = await session.get(
                f"{self._baseurl}{endpoint}", headers=self._header
            )
            result = await response.json(content_type="application/json")

            return result

    async def status(self) -> None:
        """
        To check Client connection with API

        Returns:
            int: status code
        """

        async with aiohttp.ClientSession() as session:
            response = await session.get(self._testurl, headers=self._header)

            return response.status

    async def password(self):
        """
        To access password endpoint

        Returns:
            `class`: Password Object
        """

        endpoint = await self._endpoint("password")

        return Password(password=endpoint["pass"])

    async def fact(self):
        """
        To access fact endpoint

        Returns:
            `class`: Fact Object
        """

        endpoint = await self._endpoint("fact")

        return Fact(id=endpoint["_id"], fact=endpoint["fact"], tags=endpoint["tags"])

    async def quote(self):
        """
        To access quote endpoint

        Returns:
            `class`: Quote Object
        """
        endpoint = await self._endpoint("quote")

        return Quote(
            id=endpoint["_id"],
            quote=endpoint["quote"],
            anime=endpoint["anime"],
            said=endpoint["said"],
        )

    async def waifu(self):
        """
        To access waifu endpoint

        Returns:
            `class`: Waifu Object
        """
        endpoint = await self._endpoint("waifu")

        return Waifu(
            id=endpoint["_id"],
            images=endpoint["images"],
            names=endpoint["names"],
            anime=endpoint["from"],
            statistics=endpoint["statistics"],
        )

    async def action(self, action: str):
        """
        To access action(S) endpoint

        Args:
            `action (str)` : endpoint name, list of endpoint names can be found at: https://docs.airi.kyoyo.me/endpoints/

        Returns:
            `str`: gif/image link (of action)
        """

        return await self._endpoint(action)
